@@ -25,11 +25,13 @@ THE SOFTWARE.
 
 import sys
 
-VERSION = (0, 6, 3)
+VERSION = (0, 6, 4)
 
 try:
+    from cymysql import convertersx as converters
     from cymysql.convertersx import escape_dict, escape_sequence, escape_string
 except ImportError:
+    from cymysql import converters
     from cymysql.converters import escape_dict, escape_sequence, escape_string
 from cymysql.err import Warning, Error, InterfaceError, DataError, \
      DatabaseError, OperationalError, IntegrityError, InternalError, \
