@@ -20,17 +20,12 @@ class Cursor(object):
         Do not create an instance of a Cursor yourself. Call
         connections.Connection.cursor().
         '''
-        from weakref import proxy
-        self.connection = proxy(connection)
-        self.description = None
-        self.rownumber = 0
-        self.rowcount = -1
+        self.connection = connection
         self.arraysize = 1
         self._executed = None
         self.messages = []
         self.errorhandler = connection.errorhandler
-        self._has_next = None
-        self._rows = ()
+        self._result = None
 
     def __del__(self):
         '''
@@ -38,6 +33,18 @@ class Cursor(object):
         '''
         self.close()
 
+    @property
+    def rowcount(self):
+        return -1
+
+    @property
+    def description(self):
+        return self._result.description if self._result else None
+
+    @property
+    def lastrowid(self):
+        return self._result.insert_id if self._result else None
+
     def close(self):
         '''
         Closing a cursor just exhausts all remaining data.
@@ -61,6 +68,10 @@ class Cursor(object):
         if not self._executed:
             self.errorhandler(self, ProgrammingError, "execute() first")
 
+    def _flush(self):
+        if self._result:
+            self._result.read_rest_rowdata_packet()
+
     def setinputsizes(self, *args):
         """Does nothing, required by DB API."""
 
@@ -73,7 +84,7 @@ class Cursor(object):
             self.fetchall()
         del self.messages[:]
 
-        if not self._has_next:
+        if not self._result or not self._result.has_next:
             return None
         connection = self._get_db()
         connection.next_result()
@@ -85,11 +96,12 @@ class Cursor(object):
         from sys import exc_info
 
         conn = self._get_db()
+        if hasattr(conn, "_last_execute_cursor"):
+            conn._last_execute_cursor._flush()
+
         charset = conn.charset
         del self.messages[:]
 
-        # TODO: make sure that conn.escape is correct
-
         if PYTHON3 and (not isinstance(query, str)):
             query = query.decode(charset)
         if (not PYTHON3) and isinstance(query, unicode):
@@ -107,9 +119,8 @@ class Cursor(object):
 
             query = query % escaped_args
 
-        result = 0
         try:
-            result = self._query(query)
+            self._query(query)
         except:
             exc, value, tb = exc_info()
             del tb
@@ -117,21 +128,14 @@ class Cursor(object):
             self.errorhandler(self, exc, value)
 
         self._executed = query
-        return result
+        conn._last_execute_cursor = self
 
     def executemany(self, query, args):
         ''' Run several data against one query '''
         del self.messages[:]
-        #conn = self._get_db()
-        if not args:
-            return
-        #charset = conn.charset
-        #if isinstance(query, unicode):
-        #    query = query.encode(charset)
-
-        self.rowcount = sum([ self.execute(query, arg) for arg in args ])
-        return self.rowcount
 
+        for params in args:
+            self.execute(query, params)
 
     def callproc(self, procname, args=()):
         """Execute stored procedure procname with args
@@ -182,64 +186,48 @@ class Cursor(object):
     def fetchone(self):
         ''' Fetch the next row '''
         self._check_executed()
-        if self._rows is None or self.rownumber >= len(self._rows):
+        if self._result is None:
             return None
-        result = self._rows[self.rownumber]
-        self.rownumber += 1
-        return result
+        return self._result.fetchone()
 
     def fetchmany(self, size=None):
         ''' Fetch several rows '''
         self._check_executed()
-        end = self.rownumber + (size or self.arraysize)
-        result = self._rows[self.rownumber:end]
-        if self._rows is None:
+        size = size or self.arraysize
+        if self._result is None:
             return None
-        self.rownumber = min(end, len(self._rows))
+        result = []
+        for i in range(size):
+            r = self._result.fetchone()
+            if not r:
+                break
+            result.append(r)
         return result
 
     def fetchall(self):
         ''' Fetch all the rows '''
         self._check_executed()
-        if self._rows is None:
+        if self._result is None:
             return None
-        if self.rownumber:
-            result = self._rows[self.rownumber:]
-        else:
-            result = self._rows
-        self.rownumber = len(self._rows)
+        result = []
+
+        r = self._result.fetchone()
+        while r:
+            result.append(r)
+            r = self._result.fetchone()
+
         return result
 
-    def scroll(self, value, mode='relative'):
-        self._check_executed()
-        if mode == 'relative':
-            r = self.rownumber + value
-        elif mode == 'absolute':
-            r = value
-        else:
-            self.errorhandler(self, ProgrammingError,
-                    "unknown scroll mode %s" % mode)
-
-        if r < 0 or r >= len(self._rows):
-            self.errorhandler(self, IndexError, "out of range")
-        self.rownumber = r
-
     def _query(self, q):
         conn = self._get_db()
         self._last_executed = q
         conn.query(q)
         self._do_get_result()
-        return self.rowcount
 
     def _do_get_result(self):
         conn = self._get_db()
-        self.rowcount = conn._result.affected_rows
 
-        self.rownumber = 0
-        self.description = conn._result.description
-        self.lastrowid = conn._result.insert_id
-        self._rows = conn._result.rows
-        self._has_next = conn._result.has_next
+        self._result = conn._result
 
     def __iter__(self):
         return iter(self.fetchone, None)
@@ -267,31 +255,26 @@ class DictCursor(Cursor):
     def fetchone(self):
         ''' Fetch the next row '''
         self._check_executed()
-        if self._rows is None or self.rownumber >= len(self._rows):
+        if self._result is None:
             return None
-        result = dict(zip(self._fields, self._rows[self.rownumber]))
-        self.rownumber += 1
-        return result
+        r = super(DictCursor, self).fetchone()
+        if not r:
+            return None
+        return  dict(zip(self._fields, r))
 
     def fetchmany(self, size=None):
         ''' Fetch several rows '''
         self._check_executed()
-        if self._rows is None:
+        if self._result is None:
             return None
-        end = self.rownumber + (size or self.arraysize)
-        result = [ dict(zip(self._fields, r)) for r in self._rows[self.rownumber:end] ]
-        self.rownumber = min(end, len(self._rows))
+        result = [ dict(zip(self._fields, r)) for r in super(DictCursor, self).fetchmany(size)]
         return tuple(result)
 
     def fetchall(self):
         ''' Fetch all the rows '''
         self._check_executed()
-        if self._rows is None:
+        if self._result is None:
             return None
-        if self.rownumber:
-            result = [ dict(zip(self._fields, r)) for r in self._rows[self.rownumber:] ]
-        else:
-            result = [ dict(zip(self._fields, r)) for r in self._rows ]
-        self.rownumber = len(self._rows)
+        result = [ dict(zip(self._fields, r)) for r in super(DictCursor, self).fetchall() ]
         return tuple(result)
 
